<?php

namespace Teknisa\DF\Services;

use Teknisa\DF\Modules\Entrada\Services\Dfe11000NfEntradaService;
use Teknisa\DF\Modules\Saida\Services\Dfs11000NfSaidaService;
use Teknisa\DF\Services\ReplicacaoDocumentosService;

use Doctrine\ORM\EntityManager;
use Indeterminado\Entities\Aliqtempprod; // TODO REMOVER
use Estoque\Entities\Lanctoestoq;
use Financeiro\Entities\Contareceber;
use Fiscal\Entities\Contapagar;
use Fiscal\Entities\Dadosadic;
use Fiscal\Entities\Impostonf;
use Fiscal\Entities\Itemproimpos;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Incitemp;
use Fiscal\Entities\IntgNotafiscal;
use Geral\Entities\Encargo;
use Geral\Entities\Imposto;
use Geral\Entities\Itemencargo;
use Geral\Entities\Itemproduto;
use Geral\Entities\Produto;
use Geral\Entities\Prodfili;
use Geral\Entities\Empresa;
use Geral\Entities\Endeforn;
use Geral\Entities\Endeclie;
use Geral\Entities\Endefili;
use Geral\Entities\Tipoctpagar;
use Teknisa\Libs\Util\DateUtil;
use Qualificacao\Entities\Movocorr;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\DF\Factories\Dfe57000LancamentoSimplificadoFactory;

class Dfe57000LancamentoSimplificadoService {
    const VALUE = 'VALUE';
    const VALUE_COLUMN = 'VALUE_COLUMN';
    const ESTOQUE_SEPARADOR = ' ';

    const NOVO_CODIGO_LANC_ESTOQUE  = 'LANCTOEST';

    const PROCEDURE_DELETA_NF       = 'DELETA_NF';
    const PROCEDURE_GERA_PRODUTO    = 'GERA_NOTAFISCAL_PRODUTO';
    const PROCEDURE_GERA_FECHAMENTO_SAIDA = 'GERA_NOTAFISCAL_FECHAMENTO';
    const PROCEDURE_GERA_FECHAMENTO_ENTRADA = 'GERA_FECHAMENTO_NFENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO_ENTRADA = 'GERA_NOTAFISCAL_CORPO_ENTRADA';

    protected $entityManager;
    protected $novoCodigo;
    protected $environment;
    protected $Dfe57000LancamentoSimplificadoFactory;

    public function __construct(EntityManager $entityManager, NewCode $novoCodigo, $environment, Dfe57000LancamentoSimplificadoFactory $Dfe57000LancamentoSimplificadoFactory)
    {
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->dfe57000Factory = $Dfe57000LancamentoSimplificadoFactory;
    }

    public function buscaNotas($filial, $lancamento, $dtemissao) {
        $organizacao = $this->environment->getNrOrgTrab();
        $params = array(
            "P_CDFILIAL"  => $this->montaSqlIn($filial),
            "P_CDFORNECED" => $this->montaSqlIn($lancamento),
            "P_DTINICIAL"  => '01/01/2001',
            "P_DTFINAL"    => '31/12/2019',
            "P_DTDATA" => null
        );
        
        $notas = $this->dfe57000Factory->getNotas($params);
        
        return $notas;
    }
    
    
    private function montaSqlIn($array) {
        $sql = '(';
        if($array) {
            foreach($array as $value) {
                $sql.= "'".$value."'".',';
            }
        }
        else  return null;//throw new \Exception("Não foi possivel montar o Select.");
        $size = strlen($sql);
        return substr($sql,0, $size-1).')';
    }
    
    public function buscaProdFili($filial, $arrayProdutos) {
        $organizacao = $this->environment->getNrOrgTrab();
        $arrayProdFili = [];
        foreach($arrayProdutos as $produto){
            $cdproduto = $produto['CDPRODUTO'];
            $dadosProduto = Prodfili::findByCdfilialAndCdprodutoAndNrorg($filial, $cdproduto, $organizacao);
            array_push($arrayProdFili, $dadosProduto);
        }
        return($arrayProdFili);
    }
    
    public function buscaProdEsto($arrayProdutos) {
        $organizacao = $this->environment->getNrOrgTrab();
        $arrayProdEsto = [];
        foreach($arrayProdutos as $produto){
            $cdproduto = $produto['CDPRODUTO'];
            $dadosProduto = Produto::findByCdprodutoAndNrorg($cdproduto, $organizacao);
            array_push($arrayProdEsto, $dadosProduto);
        }
        return($arrayProdEsto);
    }

    public function criaArrayNotas(array $notas) {
        
        $organizacao = $this->environment->getNrOrgTrab();
        $arrayNotas = array();
        if (!is_array($notas))
            return $arrayNotas;
        
        var_dump($notas);die;
        foreach ($notas as $key => $nota) {
            
            var_dump($nota);die;

            $produto = Produto::findOneByCdprodutoAndNrorg($item->getCdproduto(), $organizacao);
            $prodfili = Prodfili::findOneByCdfilialAndCdprodutoAndNrorg($item->getCdfilimovi(), $item->getCdproduto(), $organizacao);

            $arrayProdutos[$key]['CDFILIAL']     = $item->getCdfilial();
            $arrayProdutos[$key]['NRLANCTONF']   = $item->getNrlanctonf();
            $arrayProdutos[$key]['CDPRODUTO']    = $item->getCdproduto();
            $arrayProdutos[$key]['NRSEQITPROD']  = $item->getNrseqitprod();
            $arrayProdutos[$key]['CDPRODEMIS']   = $item->getCdprodemis();
            $arrayProdutos[$key]['DSPRODEMIS']   = $item->getDsprodemis();
            $arrayProdutos[$key]['QTITEMPROD']   = $item->getQtitemprod();
            $arrayProdutos[$key]['VRUNITITEM']   = $item->getVrunititem();
            $arrayProdutos[$key]['VRTOTAITEM']   = round($item->getVrtotaitem(), 2);
            $arrayProdutos[$key]['VRDESCITEM']   = $item->getVrdescitem();
            $arrayProdutos[$key]['CDSITUTRIBPR'] = $item->getCdsitutribpr();
            $arrayProdutos[$key]['CDFISOPEITEM'] = $item->getCdfisopeitem();
            $arrayProdutos[$key]['NRPEDPROD']    = $item->getNrpedprod();
            $arrayProdutos[$key]['NRSEQPEDPROD'] = $item->getNrseqpedprod();
            $arrayProdutos[$key]['QTVOLUITEM']   = $item->getQtvoluitem();
            $arrayProdutos[$key]['CDSERVFATU']   = $item->getCdservfatu();
            $arrayProdutos[$key]['CDSERVFILI']   = $item->getCdservfili();
            $arrayProdutos[$key]['CDALMOXARIFE'] = $item->getCdalmoxarife();
            $arrayProdutos[$key]['CDLOCALESTOQ'] = $item->getCdlocalestoq();
            $arrayProdutos[$key]['NRLOTEESTQ']   = $item->getNrloteestq();
            $arrayProdutos[$key]['NRSUBLOTE']    = $item->getNrsublote();

            $arrayProdutos[$key]['CDARVPROD'] = $produto->getCdarvprod();
            $arrayProdutos[$key]['NMPRODUTO'] = $produto->getNmproduto();
            $arrayProdutos[$key]['SGUNIDADE'] = $produto->getSgunidade();
            $arrayProdutos[$key]['VRFATOCONV'] = $produto->getVrfatoconv();
            $arrayProdutos[$key]['CDPRODESTO']   = $produto->getCdprodesto();
            
            $arrayProdutos[$key]['IDCNTRESTOQ']   = $prodfili->getIdcntrestoq();
        }

        return $arrayProdutos;
    }

    public function buscaImpostosProdutos($filial, $lancamento, $entrada_saida, $statusNota) {

        $organizacao = $this->environment->getNrOrgTrab();

        if ($entrada_saida === Notafiscal::NOTA_FISCAL_ENTRADA && $statusNota === Notafiscal::EM_DIGITACAO) {
            return $this->buscaImpostosProdutosTemp($filial, $lancamento, $organizacao);
        }

        $impostos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostosProdutos = array();

        /** @var Itemproimpos $itemproimpos */
        foreach ($impostos as $key => $itemproimpos) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($itemproimpos->getCdimposto(), $organizacao);
            $iddescrfonte = !!$imposto->getIddescrfonte() ? $imposto->getIddescrfonte() : 'R';

            $arrayImpostosProdutos[$key]['CDFILIAL'] = $itemproimpos->getCdfilial();
            $arrayImpostosProdutos[$key]['NRLANCTONF'] = $itemproimpos->getNrlanctonf();
            $arrayImpostosProdutos[$key]['NRSEQITPROD'] = $itemproimpos->getNrseqitprod();
            $arrayImpostosProdutos[$key]['CDIMPOSTO'] = $itemproimpos->getCdimposto();
            $arrayImpostosProdutos[$key]['VRALIQIMPOIT'] = $itemproimpos->getVraliqitem();
            $arrayImpostosProdutos[$key]['VRIMPOSTOIT'] = $itemproimpos->getVrimpostoit();
            $arrayImpostosProdutos[$key]['IDDESCRFONTE'] = $iddescrfonte;

            $arrayImpostosProdutos[$key]['IDVALORINFIT'] = $itemproimpos->getIdvalorinfit();

            $arrayImpostosProdutos[$key]['NMIMPOSTO'] = $imposto->getNmimposto();
            $arrayImpostosProdutos[$key]['SGIMPOSTO'] = $imposto->getSgimposto();
        }

        return $arrayImpostosProdutos;
    }

    private function buscaImpostosProdutosTemp($filial, $lancamento, $organizacao) {
        $impostos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostosProdutosTemp = array();

        /** @var Trabitproimpos $trabitproimpos */
        foreach ($impostos as $key => $trabitproimpos) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($trabitproimpos->getCdimpostoit(), $organizacao);
            $iddescrfonte = !!$imposto->getIddescrfonte() ? $imposto->getIddescrfonte() : 'R';

            $arrayImpostosProdutosTemp[$key]['CDFILIAL'] = $trabitproimpos->getCdfilimpoit();
            $arrayImpostosProdutosTemp[$key]['NRLANCTONF'] = $trabitproimpos->getNrlancimpoit();
            $arrayImpostosProdutosTemp[$key]['NRSEQITPROD'] = $trabitproimpos->getNrseqimpoit();
            $arrayImpostosProdutosTemp[$key]['CDIMPOSTO'] = $trabitproimpos->getCdimpostoit();
            $arrayImpostosProdutosTemp[$key]['VRALIQIMPOIT'] = $trabitproimpos->getVraliqimpoit();
            $arrayImpostosProdutosTemp[$key]['VRIMPOSTOIT'] = $trabitproimpos->getVrimposit();
            $arrayImpostosProdutosTemp[$key]['IDDESCRFONTE'] = $iddescrfonte;

            $arrayImpostosProdutosTemp[$key]['NMIMPOSTO'] = $imposto->getNmimposto();
            $arrayImpostosProdutosTemp[$key]['SGIMPOSTO'] = $imposto->getSgimposto();
        }

        return $arrayImpostosProdutosTemp;
    }

    public function buscaImpostos($filial, $lancamento, $entrada_saida, $statusNota) {

        $organizacao = $this->environment->getNrOrgTrab();

        if ($entrada_saida === Notafiscal::NOTA_FISCAL_ENTRADA && $statusNota === Notafiscal::EM_DIGITACAO) {
            return $this->buscaImpostosTemp($filial, $lancamento, $organizacao);
        }

        $impostos = Impostonf::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostos = array();

        /** @var Impostonf $impostoNF */
        foreach ($impostos as $key => $impostoNF) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($impostoNF->getCdimposto(), $organizacao);

            $arrayImpostos[$key]['CDFILIAL'] = $impostoNF->getCdfilial();
            $arrayImpostos[$key]['NRLANCTONF'] = $impostoNF->getNrlanctonf();
            $arrayImpostos[$key]['CDIMPOSTO'] = $impostoNF->getCdimposto();
            $arrayImpostos[$key]['VRIMPOSTONF'] = $impostoNF->getVrimpostonf();

            $arrayImpostos[$key]['NMIMPOSTO'] = $imposto->getNmimposto();
            $arrayImpostos[$key]['SGIMPOSTO'] = $imposto->getSgimposto();
        }

        return $arrayImpostos;
    }

    private function buscaImpostosTemp($filial, $lancamento, $organizacao) {
        $impostos = Trabimposnftot::findByCdfilimptotAndNrlancimptotAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostos = array();

        /** @var Trabimposnftot $impostoNF */
        foreach ($impostos as $key => $impostoNF) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($impostoNF->getCdimpostot(), $organizacao);

            $arrayImpostos[$key]['CDFILIAL'] = $impostoNF->getCdfilimptot();
            $arrayImpostos[$key]['NRLANCTONF'] = $impostoNF->getNrlancimptot();
            $arrayImpostos[$key]['CDIMPOSTO'] = $impostoNF->getCdimpostot();
            $arrayImpostos[$key]['VRIMPOSTONF'] = $impostoNF->getVrimpostot();

            $arrayImpostos[$key]['NMIMPOSTO'] = $imposto->getNmimposto();
            $arrayImpostos[$key]['SGIMPOSTO'] = $imposto->getSgimposto();
        }

        return $arrayImpostos;
    }

    public function buscaEncargos($filial, $lancamento) {
        $organizacao = $this->environment->getNrOrgTrab();
        $encargos = Itemencargo::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        $arrayEncargos = $this->criaArrayEncargos($encargos);
        return $arrayEncargos;

        // $arrayEncargos = array();
        // /** @var Itemencargo $itemencargo */
        // foreach ($encargos as $key => $itemencargo) {
        //     $encargo = Encargo::findOneByCdencargoAndNrorg($itemencargo->getCdencargo(), $organizacao);

        //     $arrayEncargos[$key]['CDFILIAL'] = $itemencargo->getCdfilial();
        //     $arrayEncargos[$key]['NRLANCTONF'] = $itemencargo->getNrlanctonf();
        //     $arrayEncargos[$key]['CDENCARGO'] = $itemencargo->getCdencargo();
        //     $arrayEncargos[$key]['VRITEMENCA'] = $itemencargo->getVritemenca();

        //     $arrayEncargos[$key]['NMENCARGO'] = $encargo->getNmencargo();
        // }

        // return $arrayEncargos;
    }

    public function criaArrayEncargos(array $encargos) {

        $arrayEncargos = array();
        $organizacao = $this->environment->getNrOrgTrab();

        foreach ($encargos as $key => $itemencargo) {
            if (!$itemencargo instanceof Itemencargo)
                throw new Exception('Encargo não encontrado.');

            $encargo = Encargo::findOneByCdencargoAndNrorg($itemencargo->getCdencargo(), $organizacao);

            $arrayEncargos[$key]['CDFILIAL'] = $itemencargo->getCdfilial();
            $arrayEncargos[$key]['NRLANCTONF'] = $itemencargo->getNrlanctonf();
            $arrayEncargos[$key]['CDENCARGO'] = $itemencargo->getCdencargo();
            $arrayEncargos[$key]['VRITEMENCA'] = $itemencargo->getVritemenca();

            $arrayEncargos[$key]['NMENCARGO'] = $encargo->getNmencargo();
            $arrayEncargos[$key]['IDENCARGO'] = $encargo->getIdencargo();
        }

        return $arrayEncargos;
    }

    public function buscaVencimentos($filial, $lancamento, $entrada_saida) {
        $organizacao = $this->environment->getNrOrgTrab();
        if($entrada_saida == 'S'){
            $vencimentos = Contareceber::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        }
        else{
            $vencimentos = Contapagar::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        }

        return $this->criaArrayVencimento($vencimentos, $entrada_saida);
    }

    public function buscaDadosAdicionais($filial, $lancamento) {
        $organizacao = $this->environment->getNrOrgTrab();
        $dadosAdicionais = Dadosadic::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $arrayDadosAdicionais = array();
        /** @var Dadosadic $dadosadic */
        foreach ($dadosAdicionais as $key => $dadosadic) {
            
            $arrayDadosAdicionais[$key]['CDFILIAL'] = $dadosadic->getCdfilial();
            $arrayDadosAdicionais[$key]['NRLANCTONF'] = $dadosadic->getNrlanctonf();
            $arrayDadosAdicionais[$key]['NRDADOADIC'] = $dadosadic->getNrdadoadic();
            $arrayDadosAdicionais[$key]['CDMENSAGEM'] = $dadosadic->getCdmensagem();
            $arrayDadosAdicionais[$key]['DSDADOADIC'] = $dadosadic->getDsdadoadic();
        }

        return $arrayDadosAdicionais;
    }

    public function procedureDeletaNf($filial, $lancamento, $corpoItem = null){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_DELETA_NF);
            $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCORPITEM', Param::PARAM_INPUT, $corpoItem, Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function fechaNotaFiscal($cabecalho, $vencimentos)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $filial = $cabecalho['CDFILIAL'];
            $organizacao = $this->environment->getNrOrgTrab();

            $filialSelecionada = ($cabecalho['IDENTRSAIDOP'] === Notafiscal::NOTA_FISCAL_ENTRADA) ? $cabecalho['CDFILIRECE'] : $cabecalho['CDFILIEMIT'] ;
            $numLancamento = $cabecalho['NRLANCTONF'];

            $this->validaFechamentoNotaUtilizaAf($cabecalho);

            $lancEstoqAnterior = Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
            if (!empty($lancEstoqAnterior)) {
                $numLancEstoque = $lancEstoqAnterior->getNrlancestq();
            }
            else {
                $numLancEstoque = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$filial, $organizacao);
            }

            $dadosProcessFilial = $this->validacoesEstoqueService->retornaDadosProcessFilial($filialSelecionada, $organizacao);
            
            
            $this->callProcedureFechamento($cabecalho, $dadosProcessFilial['DTPROCESSA'], $numLancEstoque);

            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($filial, $numLancEstoque, $organizacao);

            if (is_object($lanctoestoq))
                $this->validacoesEstoqueService->verificaIrregularidades($lanctoestoq);

            $this->validaErrosEAvisos($filial, $numLancamento);

            $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
            $this->entityManager->refresh($notaFiscal);
            $vencimentos = $this->comparaVencimentoInfoGerado($filial, $numLancamento, $notaFiscal->getVrnotafisc(), $vencimentos, $organizacao, $cabecalho['IDENTRSAIDOP']);

            $notaFiscal->setIdstatus('P');

            if($cabecalho['IDENTRSAIDOP'] === 'S'){
                if(isset($cabecalho['CDFILIEMIT']) && isset($cabecalho['CDFILIRECE'])) {
                    if($cabecalho['CDFILIEMIT'] && $cabecalho['CDFILIRECE']) {
                        $errosReplicacao = $this->replicacaoAutomatica($notaFiscal, $dadosProcessFilial['DTPROCESSA']);
                        if($errosReplicacao){
                            $connection->rollback();
                            return array('ERROS_REPLICACAO' => $errosReplicacao);
                        }
                    }
                }
            }

            $notaFiscal->persist();
            $this->entityManager->flush();
            $this->entityManager->refresh($notaFiscal);

            $connection->commit();

            $infoNotaFiscal = array(
                'CABECALHO' => array(
                    'NRLANCTONF' => $notaFiscal->getNrlanctonf(),
                    'VRNOTAFISC' => $notaFiscal->getVrnotafisc()
                ),
                'VENCIMENTO' => $vencimentos
            );

            return $infoNotaFiscal;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function replicacaoAutomatica($notafiscal, $dataProcessamento){
        $replicacao = new ReplicacaoDocumentosService($this->validacoesEstoqueService, $this->entityManager, $this->novoCodigo, $this->environment, $notafiscal, $dataProcessamento);
        return $replicacao->replicar();
    }

    public function replicacaoManual($notafiscal, $documentos, $itens) {
        $filialSelecionada = $notafiscal['CDFILIEMIT'];
        $organizacao = $this->environment->getNrOrgTrab();
        $dataProcessamento = $this->validacoesEstoqueService->retornaDadosProcessFilial($filialSelecionada, $organizacao)['DTPROCESSA'];
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($notafiscal['CDFILIAL'], $notafiscal['NRLANCTONF'], $organizacao);
        $replicacao = new ReplicacaoDocumentosService($this->validacoesEstoqueService, $this->entityManager, $this->novoCodigo, $this->environment, $notafiscal, $dataProcessamento, true);
        $errosReplicacao = $replicacao->replicar($documentos, $itens);
        if($errosReplicacao){
            return array('ERROS_REPLICACAO' => $errosReplicacao);
        }
        else {
            return array('SUCESSO' => array("SUCESSO"=>"SUCESSO"));
        }
    }

    public function procedureGeraNotaFiscalProduto(
        $filial, $numLancamento, $produto, $codEmissao, $descEmissao, $quantidade, $vrUnitario, $vrDesconto,
        $almoxarifado = null, $localEstoque = null, $nrloteestq = null, $nrsublote = null, $cdservfatu = null,
        $idOrigemProd = null, $idAtuEstProd = 'S',  $vrICMSItOrig = null, $cdMarca = null
    )
    {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_PRODUTO);
        
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $produto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODEMIS', Param::PARAM_INPUT, $codEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSPRODEMIS', Param::PARAM_INPUT, $descEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTITEMPROD', Param::PARAM_INPUT, $quantidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRUNITITEM', Param::PARAM_INPUT, $vrUnitario, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRDESCITEM', Param::PARAM_INPUT, $vrDesconto, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDFILIALPED', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPEDIDO', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRITEMPED', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCSTINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCFOPINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSUFXINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $almoxarifado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $localEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTEESTQ', Param::PARAM_INPUT, $nrloteestq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSUBLOTE', Param::PARAM_INPUT, $nrsublote, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDCONFORITEM', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPNFCREDEB', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTVOLUITEM', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAPUR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, $cdMarca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFATU', Param::PARAM_INPUT, $cdservfatu, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD', Param::PARAM_INPUT, $idOrigemProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUESTPROD', Param::PARAM_INPUT, $idAtuEstProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRICMSSTITORIG', Param::PARAM_INPUT, $vrICMSItOrig, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }

    private function procedureGeraFechamentoSaida(
        $filial, $numLancamento, $dtProcessamento, $numLancEstoque, $operador = '',
        $evolEstoque = 'S', $bloqTrans = 'N'
    )
    {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_SAIDA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EVOLESTOQ', Param::PARAM_INPUT, $evolEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDBLOQTRANS', Param::PARAM_INPUT, $bloqTrans, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

    private function procedureGeraFechamentoEntrada($filial, $numLancamento, $dtProcessamento, $numLancEstoque, $operador = '')
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $organizacao = $this->environment->getNrOrgTrab();
        $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
        $this->entityManager->refresh($notaFiscal);
        $notaFiscal->setIdstatus('P');
        $notaFiscal->persist();
        $this->entityManager->flush();
        $connection->commit();
            
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

    /* Compara o vencimento infromado pelo usuario com o vencimento gerado pela procedure.
     * Se houver diferenca do valor informado pelo usuario com o valor calculado devera ser exibido a mensagem
     * para avisar o usuário desta diferenca. A unica excecao e quando o usuario informa somente uma parcela,
     * devendo assim realizar update.
     */
    private function comparaVencimentoInfoGerado(
        $filialLancamento, $numLancamento, $valorNota, array $vencimentosInfo, $organizacao, $entrada_saida
    )
    {
        try {
            if ($entrada_saida === 'S' && count($vencimentosInfo) > 1) {
                $valorTotalInfo = $this->calcularValorTotalVencimentos($vencimentosInfo, $entrada_saida);
                $difeValorVenc = $valorNota - $valorTotalInfo;

                if ($difeValorVenc != 0)
                    throw new \Exception('A soma dos vencimentos difere do valor total da NF. Diferença ' . $difeValorVenc);
            }

            return $this->buscaVencimentos($filialLancamento, $numLancamento, $entrada_saida);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data)
    {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

    private function criaArrayVencimento($conta, $entrada_saida)
    {
        if (!is_array($conta))
            $conta = array($conta);

        $contas = array();

        if($entrada_saida == 'S'){
            foreach ($conta as $vencimento) {
                /** @var Contareceber $vencimento */
                $dtVencimento = is_object($vencimento->getDtatuavenrec()) ? $vencimento->getDtatuavenrec()->format('d/m/Y') : null;
                $contas[] = array(
                    'CDFILIAL' => $vencimento->getCdfilial(),
                    'NRLANCTONF' => $vencimento->getNrlanctonf(),
                    'NRPARCELAREC' => $vencimento->getNrparcelarec(),
                    'SEQVENCIMENTO' => $vencimento->getNrseqvenrec(), //Utiliazado para o salvamento local
                    'NRSEQVENREC' => $vencimento->getNrseqvenrec(),
                    'QTDIASVENREC' => $vencimento->getQtdiasvenrec(),
                    'DTATUAVENREC' => $dtVencimento,
                    'VRATUAREC' => $vencimento->getVratuarec(),
                    'IDSTATREC' => $vencimento->getIdstatrec()
                );
            }
        }
        else{
            foreach ($conta as $vencimento) {
                $codTipoContaPag = $vencimento->getCdtpctpagar();
                $organizacao = $this->environment->getNrOrgTrab();

                /** @var Contapagar $vencimento */
                $dtVencimento = is_object($vencimento->getDtatuavenpag()) ? $vencimento->getDtatuavenpag()->format('d/m/Y') : null;
                $dsTipoContaPag = null;
                $tipoContaPag = Tipoctpagar::findOneByCdtpctpagarAndNrorg($codTipoContaPag, $organizacao);

                if (is_object($tipoContaPag))
                    $dsTipoContaPag = $tipoContaPag->getDstpctpagar();

                $contas[] = array(
                    'CDFILIAL' => $vencimento->getCdfilial(),
                    'NRLANCTONF' => $vencimento->getNrlanctonf(),
                    'NRPARCELAPAG' => $vencimento->getNrparcelapag(),
                    'SEQVENCIMENTO' => $vencimento->getNrseqvenpag(), //Utiliazado para o salvamento local
                    'NRSEQVENPAG' => $vencimento->getNrseqvenpag(),
                    'QTDIASVENPAG' => $vencimento->getQtdiasvenpag(),
                    'DTATUAVENPAG' => $dtVencimento,
                    'VRATUAPAG' => $vencimento->getVratuapag(),
                    'IDSTATPAG' => $vencimento->getIdstatpag(),
                    'DSCONTAPAG' => $vencimento->getDscontapag(),
                    'CDTPCTPAGAR' => $codTipoContaPag,
                    'DSTPCTPAGAR' => $dsTipoContaPag
                );
            }
        }

        return $contas;
    }

    public function updateStatusNotafiscal($filial, $lancamento, $organizacao, $status){
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $notafiscal->setIdstatus($status);

        $this->entityManager->persist($notafiscal);
        $this->entityManager->flush();
    }

    public function updateVencimento(
        $entrada_saida, $filial, $numLancamento, $sequencial, $organizacao, $vrOrigem = null,
        $vrAtual = null, $dtVencimento = null, $dtOrigemVencimento = null, $status = null, $numDias = null,
        $tpContaReceber = null, $numParcelas = null, $descricao = null, $classFina = null,
        $operador = null, $ativo = null
    )
    {
        try {
            if($entrada_saida == 'S'){
                return $this->updateContareceber($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem,
                    $vrAtual, $dtVencimento, $dtOrigemVencimento, $status, $numDias,
                    $tpContaReceber, $numParcelas, $descricao, $classFina,
                    $operador, $ativo);
            }
            else{
                return $this->updateContapagar($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem,
                    $vrAtual, $dtVencimento, $dtOrigemVencimento, $status, $numDias,
                    $tpContaReceber, $numParcelas, $descricao, $classFina,
                    $operador, $ativo);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function updateContareceber($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem = null,
                                        $vrAtual = null, $dtVencimento = null, $dtOrigemVencimento = null, $status = null, $numDias = null,
                                        $tpContaReceber = null, $numParcelas = null, $descricao = null, $classFina = null,
                                        $operador = null, $ativo = null){
        /** @var Contareceber $contaReceber */
        $contaReceber = Contareceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrorg(
            $filial, $numLancamento, $sequencial, $organizacao
        );

        $values = [
            'numParcelas'        => [self::VALUE => $numParcelas,        self::VALUE_COLUMN => $contaReceber->getNrparcelarec()],
            'tpContaReceber'     => [self::VALUE => $tpContaReceber,     self::VALUE_COLUMN => $contaReceber->getCdtpctreceb()],
            'numDias'            => [self::VALUE => $numDias,            self::VALUE_COLUMN => $contaReceber->getQtdiasvenrec()],
            'vrOrigem'           => [self::VALUE => $vrOrigem,           self::VALUE_COLUMN => $contaReceber->getVrorigrec()],
            'vrAtual'            => [self::VALUE => $vrAtual,            self::VALUE_COLUMN => $contaReceber->getVratuarec()],
            'dtVencimento'       => [self::VALUE => $dtVencimento,       self::VALUE_COLUMN => $contaReceber->getDtatuavenrec()],
            'dtOrigemVencimento' => [self::VALUE => $dtOrigemVencimento, self::VALUE_COLUMN => $contaReceber->getDtorigvenrec()],
            'classFina'          => [self::VALUE => $classFina,          self::VALUE_COLUMN => $contaReceber->getCdclassfina()],
            'descricao'          => [self::VALUE => $descricao,          self::VALUE_COLUMN => $contaReceber->getDscontarec()],
            'status'             => [self::VALUE => $status,             self::VALUE_COLUMN => $contaReceber->getIdstatrec()],
            'operador'           => [self::VALUE => $operador,           self::VALUE_COLUMN => $contaReceber->getCdoperultatu()],
            'ativo'              => [self::VALUE => $ativo,              self::VALUE_COLUMN => $contaReceber->getIdativo()]
        ];

        foreach ($values as $key => $value) {
            $values[$key] = (!$value[self::VALUE]) ? $value[self::VALUE_COLUMN] : $value[self::VALUE];
        }

        /* extract() -> Verifica cada chave do array e vê se ele tem um válido nome de variável.
         *           -> Verifica por colisões com já existente variáveis.
         */
        extract($values);

        $contaReceber->setNrparcelarec($numParcelas);
        $contaReceber->setCdtpctreceb($tpContaReceber);
        $contaReceber->setQtdiasvenrec($numDias);
        $contaReceber->setVrorigrec($vrOrigem);
        $contaReceber->setVratuarec($vrAtual);
        $contaReceber->setDtatuavenrec(self::formataData($dtVencimento));
        $contaReceber->setDtorigvenrec(self::formataData($dtOrigemVencimento));
        $contaReceber->setCdclassfina($classFina);
        $contaReceber->setDscontarec($descricao);
        $contaReceber->setIdstatrec($status);
        $contaReceber->setNrorgultatu($organizacao);
        $contaReceber->setCdoperultatu($operador);
        $contaReceber->setDtultatu(DateUtil::getDataAtual());
        $contaReceber->setIdativo($ativo);

        $this->entityManager->persist($contaReceber);
        $this->entityManager->flush();

        return $contaReceber;
    }

    private function updateContapagar($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem = null,
                                        $vrAtual = null, $dtVencimento = null, $dtOrigemVencimento = null, $status = null, $numDias = null,
                                        $tpContaPagar = null, $numParcelas = null, $descricao = null, $classFina = null,
                                        $operador = null, $ativo = null){
        /** @var Contapagar $contaPagar */
        $contaPagar = Contapagar::findOneByCdfilialAndNrlanctonfAndNrseqvenpagAndNrorg(
            $filial, $numLancamento, $sequencial, $organizacao
        );

        $values = [
            'numParcelas'        => [self::VALUE => $numParcelas,        self::VALUE_COLUMN => $contaPagar->getNrparcelapag()],
            'tpContaPagar'     => [self::VALUE => $tpContaPagar,     self::VALUE_COLUMN => $contaPagar->getCdtpctpagar()],
            'numDias'            => [self::VALUE => $numDias,            self::VALUE_COLUMN => $contaPagar->getQtdiasvenpag()],
            'vrOrigem'           => [self::VALUE => $vrOrigem,           self::VALUE_COLUMN => $contaPagar->getVrorigpag()],
            'vrAtual'            => [self::VALUE => $vrAtual,            self::VALUE_COLUMN => $contaPagar->getVratuapag()],
            'dtVencimento'       => [self::VALUE => $dtVencimento,       self::VALUE_COLUMN => $contaPagar->getDtatuavenpag()],
            'dtOrigemVencimento' => [self::VALUE => $dtOrigemVencimento, self::VALUE_COLUMN => $contaPagar->getDtorigvenpag()],
            'classFina'          => [self::VALUE => $classFina,          self::VALUE_COLUMN => $contaPagar->getCdclassfina()],
            'descricao'          => [self::VALUE => $descricao,          self::VALUE_COLUMN => $contaPagar->getDscontapag()],
            'status'             => [self::VALUE => $status,             self::VALUE_COLUMN => $contaPagar->getIdstatpag()],
            'operador'           => [self::VALUE => $operador,           self::VALUE_COLUMN => $contaPagar->getCdoperultatu()],
            'ativo'              => [self::VALUE => $ativo,              self::VALUE_COLUMN => $contaPagar->getIdativo()]
        ];

        foreach ($values as $key => $value) {
            $values[$key] = (!$value[self::VALUE]) ? $value[self::VALUE_COLUMN] : $value[self::VALUE];
        }

        /* extract() -> Verifica cada chave do array e vê se ele tem um válido nome de variável.
         *           -> Verifica por colisões com já existente variáveis.
         */
        extract($values);

        $contaPagar->setNrparcelapag($numParcelas);
        $contaPagar->setCdtpctpagar($tpContaPagar);
        $contaPagar->setQtdiasvenpag($numDias);
        $contaPagar->setVrorigpag($vrOrigem);
        $contaPagar->setVratuapag($vrAtual);
        $contaPagar->setDtatuavenpag(self::formataData($dtVencimento));
        $contaPagar->setDtorigvenpag(self::formataData($dtOrigemVencimento));
        $contaPagar->setCdclassfina($classFina);
        $contaPagar->setDscontapag($descricao);
        $contaPagar->setIdstatpag($status);
        $contaPagar->setNrorgultatu($organizacao);
        $contaPagar->setCdoperultatu($operador);
        $contaPagar->setDtultatu(DateUtil::getDataAtual());
        $contaPagar->setIdativo($ativo);

        $this->entityManager->persist($contaPagar);
        $this->entityManager->flush();

        return $contaPagar;
    }

    /**
     * @param array $vencimentosInfo
     * @param       $entrada_saida
     * @return int
     */
    private function calcularValorTotalVencimentos(array $vencimentosInfo, $entrada_saida)
    {
        $valorTotalInfo = 0;
        if ($entrada_saida == 'S') {
            foreach ($vencimentosInfo as $vencInfo) {
                $valorTotalInfo += $vencInfo['VRATUAREC'];
            }
            return $valorTotalInfo;
        } else {
            foreach ($vencimentosInfo as $vencInfo) {
                $valorTotalInfo += $vencInfo['VRATUAPAG'];
            }
            return $valorTotalInfo;
        }
    }

    /**
     * @param $cabecalho
     * @param $dtProcessamento
     * @param $numLancEstoque
     * @return void
     */
    private function callProcedureFechamento($cabecalho, $dtProcessamento, $numLancEstoque)
    {
        if(is_object($dtProcessamento))
            $dtProcessamento = $dtProcessamento->format('d/m/Y');

        //@TODO verificar se precisa passar o parametro operador para a procedure

        if($cabecalho['IDENTRSAIDOP'] == 'S'){
            $this->procedureGeraFechamentoSaida(
                $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $dtProcessamento, $numLancEstoque
            );
        }

        else{
            $this->procedureGeraFechamentoEntrada(
                $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $dtProcessamento, $numLancEstoque
            );
        }

        $this->entityManager->flush();
    }

    public function calculaImpostoItemProduto($cabecalho, $item) {
        try {
            //TODO refatorarrrrrrrrrrrrr
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $filial = $cabecalho['CDFILIAL'];
            $lancamento = $cabecalho['NRLANCTONF'];
            $empresa = $cabecalho['CDEMPRESA'];
            $inscesta = $cabecalho['CDINSCESTA'];
            $tipoOperacao = $cabecalho['CDTIPOOPER'];
            $fornecedor = $cabecalho['CDFORNECED'];
            $dtEntrada = $cabecalho['DTENTRSAID'];
            $entrada_saida = $cabecalho['IDENTRSAIDOP'];
            $idorigdestnf = $cabecalho['IDORIGDESTNF'];
            $filialRece = $cabecalho['CDFILIRECE'];
            // $cdfiliemit = $cabecalho['CDFILIEMIT'];
            $cdfiliemit = null;
            $cliente = $cabecalho['CDCLIENTE'];

            $organizacao = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();
            $dtInclusao = DateUtil::getDataAtual();

            $seqItem = $item['NRSEQITPROD'];
            $cdarvprod = $item['CDARVPROD'];
            $valorProduto = $item['VRTOTAITEM'];

            $impostosTemporarios = array();
            $trabitproimpostos = array();
            $itensImpostos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
            if (is_array($itensImpostos)) {
                foreach ($itensImpostos as $itemImposto) {
                    $imposto = $itemImposto->getCdimposto();
                    $seqItem = $itemImposto->getNrseqitprod();

                    $vraliqitem = $itemImposto->getVraliqitem();
                    $vrimpostoit = $itemImposto->getVrimpostoit();
                    $alteracaoUser = $itemImposto->getIdavisimpit();

                    $trabitproimpostos[] = $this->novoTrabitproimpos(
                        $filial, $lancamento, $seqItem, $imposto, $organizacao, $operador, $dtInclusao,
                        $vrimpostoit, $vraliqitem, $alteracaoUser
                    );
                    $itemImposto->delete();
                }
                $this->entityManager->flush();
            }

            if (array_key_exists('__generate_taxes', $item) && $item['__generate_taxes']) {
                $trabitproimpos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrseqimpoitAndNrorg($filial, $lancamento, $seqItem, $organizacao);
                if (is_array($trabitproimpos)) {
                    foreach ($trabitproimpos as $trabImposto) {
                        $trabImposto->delete();
                    }
                }
                $this->entityManager->flush();

                $grupofiscal = $this->callConsultaGrupofiscal(
                    $idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao
                );

                $idinterest = $this->getIdinterest(
                    $empresa, $inscesta, $cliente, $fornecedor, $filial, $idorigdestnf, $organizacao
                );

                $cdgrpfiscsv = '';

                $this->callProcedureImpostoEntrada(
                    $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
                    $lancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
                    $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
                    $cliente, $valorProduto
                );

                $aliqtempprod = Aliqtempprod::findByNraliqtempAndNrorg($filial . $lancamento, $organizacao);
                foreach ($aliqtempprod as $aliq) {
                    $impostosTemporarios[$aliq->getCdimpotemp()] = array(
                        'CDIMPOTEMP' => $aliq->getCdimpotemp(),
                        'VRALIQIMPOIT' => $aliq->getVraliqutmp()
                    );
                    $aliq->delete();
                }

                $incitemp = Incitemp::findByNrincitempAndNrorg($filial . $lancamento, $organizacao);
                foreach ($incitemp as $inci) {
                    $impostosTemporarios[$inci->getCdimpotemp()]['VRIMPOSTOIT'] = 0;
                    $inci->delete();
                }

                foreach ($impostosTemporarios as $imposto) {
                    $cdimposto = $imposto['CDIMPOTEMP'];
                    $vrimpostoit = $imposto['VRIMPOSTOIT'];
                    $vraliqitem = $imposto['VRALIQIMPOIT'];
                    $alteracaoUser = 'N';

                    $this->novoTrabitproimpos(
                        $filial, $lancamento, $seqItem, $cdimposto, $organizacao, $operador, $dtInclusao,
                        $vrimpostoit, $vraliqitem, $alteracaoUser
                    );
                }
                $this->entityManager->flush();
            } else {
                foreach ($trabitproimpostos as $imposto) {
                    $impostosTemporarios[] = array(
                        'CDIMPOTEMP' => $imposto->getCdimpostoit(),
                        'VRALIQIMPOIT' => $imposto->getVraliqimpoit(),
                        'VRIMPOSTOIT' => $imposto->getVrimposit()
                    );
                }
            }

            $connection->commit();

            foreach ($impostosTemporarios as &$impos) {
                $cdimposto = $impos['CDIMPOTEMP'];
                $imposto = Imposto::findOneByCdimpostoAndNrorg($cdimposto, $organizacao);
                $impos['NMIMPOSTO'] = $imposto->getNmimposto();
                $impos['SGIMPOSTO'] = $imposto->getSgimposto();
            }

            return $impostosTemporarios;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function callProcedureImpostoEntrada(
        $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
        $seqLancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
        $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
        $cliente, $valorProduto
    ) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'IMPOSTO_ENTRADA_PRODUTO');
        $procedure->addParam(new Param('P_EMPRESA', Param::PARAM_INPUT, $empresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA', Param::PARAM_INPUT, $inscesta, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER', Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $seqLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID', Param::PARAM_INPUT, $dtEntrada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAID', Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC', Param::PARAM_INPUT, $grupofiscal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDINTEREST', Param::PARAM_INPUT, $idinterest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD', Param::PARAM_INPUT, $cdarvprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRPFISCSV', Param::PARAM_INPUT, $cdgrpfiscsv, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGDESTNF', Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE', Param::PARAM_INPUT, $filialRece, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT', Param::PARAM_INPUT, $cdfiliemit, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $cliente, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPRODUTO', Param::PARAM_INPUT, $valorProduto, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

    public function callConsultaGrupofiscal($idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CONSULTA_GRUPOFISCAL');
        $procedure->addParam(new Param("P_IDORIGDESTNF", Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDORIGDESTNF", Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIFATUR", Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDENTRSAIDOP", Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER", Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC", Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 10));
        $output = $procedure->execute();

        return $output['P_CDGRUPOFISC'];
    }

    public function getIdinterest($empresa, $cdinscesta, $cliente, $fornecedor, $filial, $idorigdestnf, $organizacao) {
        $idinterest = 'N';

        $endeEmpresa = Empresa::getRepository()->getEstadoPais(
            $empresa, $cdinscesta, $organizacao
        );

        switch ($idorigdestnf) {
            case 'F':
                $endeForn = Endeforn::getRepository()->getEstadoPais($fornecedor, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeForn['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeForn['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
            case 'C':
                $endeClie = Endeclie::getRepository()->getEstadoPais($cliente, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeClie['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeClie['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
            case 'L':
                $endeFili = Endefili::getRepository()->getEstadoPais($filial, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeFili['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeFili['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
        }
        return $idinterest;
    }

    public function novoTrabitproimpos(
        $filial, $lancamento, $seqItem, $imposto, $organizacao, $operador, $dtInclusao,
        $vrimpostoit, $vraliqitem, $alteracaoUser, $nrseqitpro = 1, $vrcontimpoit = 0,
        $vrcontimpoit = 0, $vrredbcimpoit = 0, $vrbcalcimpoit = 0, $vrpdedimpoit = 0,
        $idimpdesotrabit = 'N', $idmotdesotrabit = '9', $iddifaliqcuspti = 'N'
    ) {
        $itemImpostoTemp = new Trabitproimpos();

        $cdfiscimpoit = ($vrimpostoit > 0)? 1 : 2;

        $itemImpostoTemp->setCdfilimpoit($filial);
        $itemImpostoTemp->setNrlancimpoit($lancamento);
        $itemImpostoTemp->setNrseqimpoit($seqItem);
        $itemImpostoTemp->setCdimpostoit($imposto);
        $itemImpostoTemp->setNrseqitpro($nrseqitpro);
        $itemImpostoTemp->setNrorg($organizacao);

        $itemImpostoTemp->setCdfiscimpoit($cdfiscimpoit);

        $itemImpostoTemp->setCdoperinclusao($operador);
        $itemImpostoTemp->setDtinclusao($dtInclusao);

        $itemImpostoTemp->setVrcontimpoit($vrcontimpoit);
        $itemImpostoTemp->setVrredbcimpoit($vrredbcimpoit);
        $itemImpostoTemp->setVrbcalcimpoit($vrbcalcimpoit);
        $itemImpostoTemp->setVrpdedimpoit($vrpdedimpoit);

        $itemImpostoTemp->setIdimpdesotrabit($idimpdesotrabit);
        $itemImpostoTemp->setIdmotdesotrabit($idmotdesotrabit);
        $itemImpostoTemp->setIddifaliqcuspti($iddifaliqcuspti);

        $itemImpostoTemp->setVrimposit($vrimpostoit);
        $itemImpostoTemp->setVraliqimpoit($vraliqitem);
        $itemImpostoTemp->setIdavisimpoit($alteracaoUser);
        $itemImpostoTemp->persist();

        return $itemImpostoTemp;
    }

    public function salvaItensAf(array $afSelecionadas) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $organizacao = $this->environment->getNrOrgTrab();
            $nrseqitprod = 0;
            $arrayProdutos = array();

            foreach ($afSelecionadas as $af) {
                $filial = $af['CDFILIAL'];
                $seqLancamento = $af['NRLANCTONF'];

                $filialDestAf = $af['CDFILDEST'];
                $nrAutoForn = $af['NRAUTOFORN'];
                $itensAf = \Geral\Entities\Itemautf::findByCdfildestAndNrautoforn($filialDestAf, $nrAutoForn);
                if (is_array($itensAf)) {
                    foreach ($itensAf as $item) {
                        $produto = Produto::findOneByCdprodutoAndNrorg($item->getCdproduto(), $organizacao);

                        $nrseqitprod = str_pad(++$nrseqitprod, 4, '0', STR_PAD_LEFT);
                        $cdproduto = $item->getCdproduto();
                        $dtprogentaf = $item->getDtprogentaf()->format('d/m/Y');
                        $qtitemprod = floatval($item->getQtprogentaf()) - floatval($item->getQtefetentraf());

                        $cdprodemis = $cdproduto;
                        $dsprodemis = $produto->getNmproduto();

                        $vrbrutitem = $item->getVrpreuniaf();
                        $vrunititem = $item->getVrpreuniaf();
                        $vrdescitem = floatval($item->getVrdescitmaf());
                        $vrtotaitem = $vrunititem * $qtitemprod;

                        $arrayProdutos[] = $this->salvaItemproduto(
                            $filial, $seqLancamento, $nrseqitprod, $cdproduto, $cdprodemis,
                            $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
                            $vrtotaitem
                        );
                    }
                }
            }

            //$connection->commit();
            $arrayProdutos = $this->criaArrayProdutos($arrayProdutos);

            // Inserir na NFRELACAF o relacionamento da af com a nota

            return $arrayProdutos;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function salvaItemproduto(
        $filial, $seqLancamento, $nrseqitprod, $cdproduto, $cdprodemis,
        $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
        $vrtotaitem, $cdalmoxarife = self::ESTOQUE_SEPARADOR, $cdlocalestoq = self::ESTOQUE_SEPARADOR,
        $nrloteestq = self::ESTOQUE_SEPARADOR, $nrsublote = self::ESTOQUE_SEPARADOR,
        $idatuvrestoq = 'S', $vrcontitem = 0, $idconforitem = 'S'
    ) {
        try {
            $organizacao = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();
            $dtInclusao = DateUtil::getDataAtual();

            $itemproduto = new Itemproduto();
            $itemproduto->setCdfilial($filial);
            $itemproduto->setNrlanctonf($seqLancamento);
            $itemproduto->setNrseqitprod($nrseqitprod);
            $itemproduto->setCdproduto($cdproduto);
            $itemproduto->setCdprodemis($cdprodemis);
            $itemproduto->setDsprodemis($dsprodemis);
            $itemproduto->setQtitemprod($qtitemprod);
            $itemproduto->setVrbrutitem($vrbrutitem);
            $itemproduto->setVrunititem($vrunititem);
            $itemproduto->setVrdescitem($vrdescitem);
            $itemproduto->setVrtotaitem($vrtotaitem);
            $itemproduto->setCdalmoxarife($cdalmoxarife);
            $itemproduto->setCdlocalestoq($cdlocalestoq);
            $itemproduto->setNrloteestq($nrloteestq);
            $itemproduto->setNrsublote($nrsublote);
            $itemproduto->setIdatuvrestoq($idatuvrestoq);
            $itemproduto->setVrcontitem($vrcontitem);
            $itemproduto->setIdconforitem($idconforitem);

            $itemproduto->setNrorg($organizacao);
            $itemproduto->setNrorginclusao($organizacao);
            $itemproduto->setNrorgultatu($organizacao);
            $itemproduto->setCdoperinclusao($operador);
            $itemproduto->setCdoperultatu($operador);
            $itemproduto->setDtinclusao(DateUtil::getDataAtual());
            $itemproduto->setDtultatu(DateUtil::getDataAtual());

            $itemproduto->persist();
            $this->entityManager->flush();

            return $itemproduto;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaExclusaoNotaUtilizaAf($filial, $lancamento) {
        $lancamentoSaidaService = Dfs11000NfSaidaService::instanceService(
            $this->entityManager, $this->novoCodigo, $this->environment,
            $filial, $lancamento, $this->environment->getNrOrgTrab()
        );

        return $lancamentoSaidaService->atualizarStatusAFdevolucaoExclusao();
    }

    private function validaFechamentoNotaUtilizaAf(array $cabecalho) {
        $entradaSaida = $cabecalho['IDENTRSAIDOP'];

        $organizacao = $this->environment->getNrOrgTrab();
        $filial = $cabecalho['CDFILIAL'];
        $lancamento = $cabecalho['NRLANCTONF'];

        if ($entradaSaida === Notafiscal::NOTA_FISCAL_SAIDA){
            $lancamentoSaidaService = Dfs11000NfSaidaService::instanceService(
                $this->entityManager, $this->novoCodigo, $this->environment,
                $filial, $lancamento, $organizacao
            );

            return $lancamentoSaidaService->atualizarStatusAFdevolucaoAlteracao($cabecalho['IDOPERDEVONF']);
        }

        $lancamentoEntradaService = Dfe11000NfEntradaService::instanceService(
            $this->entityManager, $this->novoCodigo, $this->environment,
            $filial, $lancamento
        );

        $lancamentoEntradaService->atualizaStatusAF($filial, $lancamento, $organizacao, 'I');
    }

    private function validaErrosEAvisos($filial, $lancamento) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $confereErrosAvisosNota = new ConfereErrosAvisosNota($this->entityManager, $filial, $lancamento);
            $errosAvisos = $confereErrosAvisosNota->confereErros();

            if (count($errosAvisos) > 0)
                throw new \Exception(json_encode($errosAvisos, JSON_FORCE_OBJECT));

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function geraCorpoNotaEntrada($cdfilial, $cdfilirece, $cdempresa, $cdtipooper, $cdserieentr, $cdcliente, $sistema, $atualizaestoque, 
        $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq,
        $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $vrnotafisc, $nrlancorig, $dtlancmennf, $dtemissao, $nrnotafisc, $nracessonfe, $cdmododocfisc) {
        
        $dtemissao  = str_replace("-", "/", $dtemissao);
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_NOTA_CORPO_ENTRADA);

        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $cdfilirece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIEENTR',     Param::PARAM_INPUT, $cdserieentr,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRNOTAFISC',      Param::PARAM_INPUT, $vrnotafisc,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRNOTAFISC',      Param::PARAM_INPUT, $nrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTEMISSAO',       Param::PARAM_INPUT, $dtemissao,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRACESSONFE',     Param::PARAM_INPUT, $nracessonfe,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMODDOCFISC',    Param::PARAM_INPUT, $cdmododocfisc,     Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
        
    }
    
    private function getDataComBarras($data) {
        $data = str_replace("/", "-", $data);
        return date('d-m-Y', strtotime($data));
    }
}
